aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--include/config.h1
-rw-r--r--sway/commands.c10
-rw-r--r--sway/config.c36
3 files changed, 43 insertions, 4 deletions
diff --git a/include/config.h b/include/config.h
index 631c8178..b9ef340b 100644
--- a/include/config.h
+++ b/include/config.h
@@ -99,6 +99,7 @@ bool read_config(FILE *file, bool is_active);
*/
char *do_var_replacement(char *str);
int output_name_cmp(const void *item, const void *data);
+void merge_output_config(struct output_config *dst, struct output_config *src);
/** Sets up a WLC output handle based on a given output_config.
*/
void apply_output_config(struct output_config *oc, swayc_t *output);
diff --git a/sway/config.c b/sway/config.c
index 67b442ad..aa4675ce 100644
--- a/sway/config.c
+++ b/sway/config.c
@@ -271,6 +271,42 @@ int output_name_cmp(const void *item, const void *data) {
return strcmp(output->name, name);
}
+void merge_output_config(struct output_config *dst, struct output_config *src) {
+ if (src->name) {
+ if (dst->name) {
+ free(dst->name);
+ }
+ dst->name = strdup(src->name);
+ }
+ if (src->enabled != -1) {
+ dst->enabled = src->enabled;
+ }
+ if (src->width != -1) {
+ dst->width = src->width;
+ }
+ if (src->height != -1) {
+ dst->height = src->height;
+ }
+ if (src->x != -1) {
+ dst->x = src->x;
+ }
+ if (src->y != -1) {
+ dst->y = src->y;
+ }
+ if (src->background) {
+ if (dst->background) {
+ free(dst->background);
+ }
+ dst->background = strdup(src->background);
+ }
+ if (src->background_option) {
+ if (dst->background_option) {
+ free(dst->background_option);
+ }
+ dst->background_option = strdup(src->background_option);
+ }
+}
+
void apply_output_config(struct output_config *oc, swayc_t *output) {
if (oc && oc->width > 0 && oc->height > 0) {
output->width = oc->width;
diff --git a/sway/commands.c b/sway/commands.c
index f891792f..452f3c34 100644
--- a/sway/commands.c
+++ b/sway/commands.c
@@ -804,12 +804,14 @@ static struct cmd_results *cmd_output(int argc, char **argv) {
i = list_seq_find(config->output_configs, output_name_cmp, name);
if (i >= 0) {
// replace existing config
struct output_config *oc = config->output_configs->items[i];
- list_del(config->output_configs, i);
- free_output_config(oc);
+ merge_output_config(oc, output);
+ free_output_config(output);
+ output = oc;
+ } else {
+ list_add(config->output_configs, output);
}
- list_add(config->output_configs, output);
sway_log(L_DEBUG, "Config stored for output %s (enabled:%d) (%d x %d @ %d, %d) (bg %s %s)",
output->name, output->enabled, output->width,
@@ -804,12 +804,14 @@ static struct cmd_results *cmd_output(int argc, char **argv) {
i = list_seq_find(config->output_configs, output_name_cmp, name);
if (i >= 0) {
- // replace existing config
+ // merge existing config
struct output_config *oc = config->output_configs->items[i];